"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cache = void 0;
class Cache extends Map {
    limit;
    #sweeper;
    constructor(limit, entries) {
        super();
        this.#sweeper = null;
        if (Number.isNaN(Number(limit)) &&
            (Array.isArray(limit) ||
                (typeof limit === "object" && Symbol.iterator in limit))) {
            entries = limit;
            limit = Infinity;
        }
        else if (typeof limit === "object" && "maxSize" in limit) {
            const options = limit;
            this.#sweeper = {
                lifetime: options.sweeper?.lifetime,
                keep: options.sweeper?.keepFilter,
                filter: options.sweeper?.filter,
            };
            limit = limit.maxSize;
        }
        if (entries) {
            for (const [key, value] of entries) {
                this.set(key, value);
            }
        }
        this.limit = Number(limit);
    }
    set(key, value) {
        super.set(key, value);
        if (this.size > this.limit) {
            while (this.size > this.limit) {
                this.delete(this.keys().next().value);
            }
        }
        if (this.#sweeper && this.#sweeper.filter && !this.#sweeper.keep?.(value)) {
            setTimeout(() => this.sweep(this.#sweeper.filter), this.#sweeper.lifetime || 120_000);
        }
        return this;
    }
    forEach(fn) {
        for (const [key, value] of this.entries()) {
            fn(value, key, this);
        }
    }
    filter(fn) {
        const filteredCache = new Cache(this.limit);
        for (const [key, value] of this.entries()) {
            if (fn(value, key, this)) {
                filteredCache.set(key, value);
            }
        }
        return filteredCache;
    }
    find(fn) {
        for (const [key, value] of this) {
            if (fn(value, key, this)) {
                return [key, value];
            }
        }
        return undefined;
    }
    every(fn) {
        return [...this.entries()].every(([key, value]) => fn(value, key, this));
    }
    some(fn) {
        return [...this.entries()].some(([key, value]) => fn(value, key, this));
    }
    map(fn) {
        const mappedCachedMap = new Cache(this.limit);
        for (const [key, value] of this) {
            const e = fn(value, key, this);
            if (typeof e === "object" && "key" in e && "value" in e) {
                mappedCachedMap.set(e.key, e.value);
            }
        }
        return mappedCachedMap;
    }
    first(amount) {
        if (amount && amount > 0) {
            amount = Math.min(this.size, amount);
            const arr = Array.from({ length: amount }, () => this.values().next().value);
            return amount > this.limit ? arr.splice(this.limit, amount) : arr;
        }
        else if (amount && amount < 0) {
            return this.last(amount * -1);
        }
        return this.values().next().value;
    }
    last(amount) {
        const arr = [...this.values()];
        if (amount && amount < 0) {
            return this.first(amount * -1);
        }
        else if (amount || amount === 0) {
            return arr.slice(-amount);
        }
        return arr[arr.length - 1];
    }
    at(index) {
        index = Math.floor(index);
        const arr = [...this.values()];
        return arr.at(index);
    }
    reverse() {
        return new Cache(this.limit, [...this.entries()].reverse());
    }
    indexOf(value) {
        const arr = [...this.values()];
        return arr.indexOf(value);
    }
    reduce(fn, initialValue) {
        return [...this.entries()].reduce(fn, initialValue);
    }
    keyOf(value) {
        const indexOf = this.indexOf(value);
        const arr = [...this.keys()];
        return arr[indexOf];
    }
    sweep(fn) {
        const previousSize = this.size;
        for (const [key, value] of this) {
            if (fn(value, key, this)) {
                this.delete(key);
            }
        }
        return previousSize - this.size;
    }
    partition(fn) {
        const [cachedTrue, cachedFalse] = [new Cache(), new Cache()];
        for (const [key, value] of this) {
            if (fn(value, key, this)) {
                cachedTrue.set(key, value);
            }
            else {
                cachedFalse.set(key, value);
            }
        }
        return [cachedTrue, cachedFalse];
    }
}
exports.Cache = Cache;
