"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = exports.InteractionClient = exports.BaseClient = void 0;
const CacheManager_1 = require("../cache/CacheManager");
const Interaction_1 = require("../resources/Interaction");
const User_1 = require("../resources/User");
const index_1 = require("../utils/index");
const node_events_1 = __importDefault(require("node:events"));
const interactions_1 = require("@darkcord/interactions");
const rest_1 = require("@darkcord/rest");
const PluginManager_1 = require("../utils/PluginManager");
const Resolvable_1 = require("../utils/Resolvable");
const Application_1 = require("../resources/Application");
const WebSocket_1 = require("./WebSocket");
class BaseClient extends node_events_1.default {
    /** Client rest to make requests */
    rest;
    /**
     * Client options
     */
    options;
    /**
     * Application of this client
     */
    application;
    /**
     * Client has ready
     */
    isReady;
    /**
     * Time that the bot was ready
     */
    readyAt;
    constructor(options) {
        super();
        if (!options || typeof options !== "object") {
            options = {};
        }
        this.isReady = false;
        this.options = options;
        this.rest = new rest_1.Rest();
    }
    /**
     * Decorator to listen event
     * @param event the event for target
     * @returns
     */
    listen(event) {
        return (target, name) => {
            this.on(event, target[name]);
        };
    }
    connect() { }
}
exports.BaseClient = BaseClient;
class InteractionClient extends BaseClient {
    publicKey;
    webserver;
    cache;
    user;
    /**
     * Channels cache
     */
    channels;
    /**
     * Client guilds
     */
    guilds;
    /**
     * Client roles
     */
    roles;
    /**
     * Client emojis
     */
    emojis;
    /**
     * Client threads
     */
    threads;
    /**
     * Client users
     */
    users;
    constructor(publicKey, options) {
        super(options);
        this.publicKey = publicKey;
        if (!options || typeof options !== "object") {
            throw new TypeError("Invalid client options");
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!options.webserver || typeof options.webserver !== "object") {
            throw new TypeError("Invalid webserver options");
        }
        if (!publicKey || typeof publicKey !== "string")
            throw (0, index_1.MakeError)({
                name: "InvalidPublicKey",
                message: "Invalid public key was provided.",
            });
        if (options.rest?.token &&
            (!options.rest.token || typeof options.rest.token !== "string")) {
            throw (0, index_1.MakeError)({
                name: "InvalidToken",
                message: "Invalid token was provided.",
            });
        }
        this.user = null;
        const token = options.rest?.token?.replace(/^(Bot|Bearer)\s*/i, "");
        if (token) {
            this.rest.setToken(token);
        }
        this.webserver = new interactions_1.WebServer({
            ...options.webserver,
            publicKey,
            token: token,
        });
        this.options.resolvePartialData = Boolean(options.resolvePartialData);
        const cache = new CacheManager_1.CacheManager(this);
        this.cache = cache;
        this.channels = cache.channels;
        this.guilds = cache.guilds;
        this.roles = cache.roles;
        this.emojis = cache.emojis;
        this.threads = cache.threads;
        this.users = cache.users;
        this.webserver.on("listen", async () => {
            this.readyAt = Date.now();
            this.isReady = true;
            this.emit("connect");
        });
        this.webserver.on("interactionDataReceived", async (body, res) => {
            let interaction = Interaction_1.Interaction.from({ ...body, client: this }, res);
            if (this.options.resolvePartialData) {
                interaction = await Resolvable_1.Resolvable.resolvePartialHTTPInteractionValues(interaction, this);
            }
            this.emit("interactionCreate", interaction);
        });
    }
    async connect() {
        if (this.options.rest?.token) {
            const rawApplication = await this.rest.getCurrentApplication();
            this.application = new Application_1.ClientApplication({
                ...rawApplication,
                client: this,
            });
            try {
                const rawUser = await this.rest.getUser(rawApplication.id);
                this.user = new User_1.ClientUser({
                    ...rawUser,
                    client: this,
                });
            }
            catch {
                // Ignore
            }
        }
        await this.webserver.listen();
    }
}
exports.InteractionClient = InteractionClient;
class Client extends BaseClient {
    /**
     * Client token
     */
    token;
    /**
     * Client application id
     */
    applicationId;
    /**
     * Client application flags
     */
    applicationFlags;
    /**
     * The client cache
     */
    cache;
    /**
     * Client websocket manager for shards
     */
    websocket;
    /**
     * Client user
     */
    user;
    /**
     * Plugin manager for library plugins
     */
    pluginManager;
    /**
     * Channels cache
     */
    channels;
    /**
     * Client guilds
     */
    guilds;
    /**
     * Client roles
     */
    roles;
    /**
     * Client emojis
     */
    emojis;
    /**
     * Client threads
     */
    threads;
    /**
     * Client users
     */
    users;
    constructor(token, options) {
        super(options);
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!options || typeof options !== "object") {
            throw new TypeError("Invalid client options");
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!options.gateway || typeof options.gateway !== "object") {
            throw new TypeError("Invalid gateway options");
        }
        options.gateway.compress = Boolean(options.gateway.compress);
        if (!options.gateway.encoding) {
            let Erlpack;
            try {
                Erlpack = Boolean(require("erlpack"));
            }
            catch {
                Erlpack = false;
            }
            options.gateway.encoding = Erlpack ? "etf" : "json";
        }
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        if (!options.gateway.intents) {
            throw (0, index_1.MakeError)({
                name: "MissingIntents",
                message: "Valid intents must be provided for the client",
            });
        }
        if (Array.isArray(options.gateway.intents)) {
            options.gateway.intents = (0, index_1.bitsArrayToBits)(options.gateway.intents);
        }
        if (options.gateway.totalShards &&
            typeof options.gateway.totalShards !== "number") {
            options.gateway.totalShards = undefined;
        }
        options.gateway.disabledEvents ??= [];
        this.pluginManager = new PluginManager_1.PluginManager(this);
        if (options.plugins?.length) {
            for (const plugin of options.plugins) {
                this.pluginManager.load(plugin);
            }
        }
        this.options = Object.assign(options, 
        // @ts-ignore
        super.options);
        if (!token || typeof token !== "string")
            throw (0, index_1.MakeError)({
                name: "InvalidToken",
                message: "Invalid token was provided.",
            });
        this.rest.setToken((this.token = token.replace(/^(Bot|Bearer)\s*/i, "")));
        const cache = new CacheManager_1.CacheManager(this);
        this.cache = cache;
        this.channels = cache.channels;
        this.guilds = cache.guilds;
        this.roles = cache.roles;
        this.emojis = cache.emojis;
        this.threads = cache.threads;
        this.users = cache.users;
        this.websocket = new WebSocket_1.WebSocket(this);
        // This is set in ready
        this.user = null;
        this.application = null;
    }
    connect() {
        return this.websocket.connect();
    }
    disconnect() {
        return this.websocket.disconnect();
    }
    /**
     * Sets the status of the client user.
     * @param data Status payload
     * @returns
     */
    setStatus(data) {
        this.websocket.setStatus(data);
        return data;
    }
    /**
     * Sets the status of the client user in specified shard.
     * @param data Status payload
     * @returns
     */
    setShardStatus(shardId, data) {
        this.websocket.setShardStatus(String(shardId), data);
        return {
            shardId,
            status: data,
        };
    }
    get shards() {
        return this.websocket.shards;
    }
}
exports.Client = Client;
