"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocket = void 0;
/* eslint-disable @typescript-eslint/no-unnecessary-condition */
const Constants_1 = require("../utils/Constants");
const utils_1 = require("@darkcord/utils");
const ws_1 = require("@darkcord/ws");
const EventSource_1 = require("../gateway/EventSource");
const Cache_1 = require("../cache/Cache");
class WebSocket {
    client;
    shards;
    #queue;
    #buckets;
    totalShards;
    maxConcurrency;
    #connectTimeout;
    constructor(client) {
        this.client = client;
        this.shards = new Cache_1.Cache();
        this.#queue = [];
        this.#buckets = {};
        this.#connectTimeout = null;
    }
    get ping() {
        let ping = 0;
        for (const shard of this.shards.values()) {
            ping += shard.ping;
        }
        return ping / this.shards.size;
    }
    async handleShards() {
        if (this.#queue.length === 0) {
            return;
        }
        const maxConcurrency = this.maxConcurrency || this.client.options.gateway.concurrency;
        for (const shard of this.#queue) {
            const ratelimitKey = Number(shard.shardId) % maxConcurrency ?? 0;
            const lastConnect = this.#buckets[ratelimitKey] ?? 0;
            if (!shard.sessionId && Date.now() - lastConnect < 5_000) {
                continue;
            }
            if (this.shards.some((s) => {
                const _ratelimitKey = Number(shard.shardId) % maxConcurrency ?? 0;
                return (s.status === ws_1.GatewayStatus.Connecting &&
                    _ratelimitKey === ratelimitKey);
            })) {
                continue;
            }
            // Handling shard
            await this.handleShard(shard);
            this.#buckets[ratelimitKey] = Date.now();
        }
        this.client.emit("connect");
    }
    async handleShard(gatewayShard) {
        const _existing = this.#queue.findIndex((s) => s.shardId === gatewayShard.shardId);
        if (_existing !== -1) {
            this.#queue.splice(_existing, 1);
        }
        const id = gatewayShard.shardId;
        this.client.emit(Constants_1.Events.Debug, `Starting connecting Shard ${id}`);
        gatewayShard.on(Constants_1.ShardEvents.Connect, () => this.client.emit(Constants_1.Events.ShardConnect, id));
        gatewayShard.on(Constants_1.ShardEvents.Ready, () => this.client.emit(Constants_1.Events.ShardReady, id));
        gatewayShard.on(Constants_1.ShardEvents.PreReady, () => this.client.emit(Constants_1.Events.ShardPreReady, id));
        gatewayShard.on(Constants_1.ShardEvents.Reconnecting, () => this.client.emit(Constants_1.Events.Reconnecting));
        gatewayShard.on(Constants_1.ShardEvents.Ping, (ping) => this.client.emit(Constants_1.Events.ShardPing, ping, id));
        gatewayShard.on(Constants_1.ShardEvents.Hello, () => this.client.emit(Constants_1.Events.ShardHello, id));
        gatewayShard.on(Constants_1.ShardEvents.ReconnectRequired, () => this.client.emit(Constants_1.Events.ShardReconnectRequired, id));
        gatewayShard.on(Constants_1.ShardEvents.Debug, (message) => this.client.emit(Constants_1.Events.ShardDebug, message));
        this.shards.set(id, gatewayShard);
        await gatewayShard.connect();
    }
    async connect() {
        this.client.emit(Constants_1.Events.Debug, "Starting Shards...");
        let totalShards = this.client.options.gateway.totalShards;
        let maxConcurrency = this.client.options.gateway.concurrency;
        let gateway;
        if (!totalShards || !maxConcurrency) {
            // Fetch gateway
            gateway = await this.client.rest
                .getGateway()
                .catch((err) => {
                throw err.status === 401 ? utils_1.InvalidTokenError : err;
            });
        }
        if (!totalShards) {
            totalShards = gateway.shards;
            this.client.emit(Constants_1.Events.Debug, `Using recommend shards count provided by Discord: ${gateway.shards}`);
        }
        if (!maxConcurrency) {
            maxConcurrency = gateway.session_start_limit.max_concurrency;
            this.client.emit(Constants_1.Events.Debug, `Using max concurrency provided by Discord: ${gateway.session_start_limit.max_concurrency}`);
        }
        this.totalShards ??= totalShards;
        this.maxConcurrency ??= maxConcurrency;
        const compress = this.client.options.gateway.compress;
        for (let id = 0; id < totalShards; id++) {
            const shard = new ws_1.GatewayShard(this.client, new EventSource_1.EventSource(), {
                compress,
                shardId: id.toString(),
                encoding: this.client.options.gateway.encoding,
            });
            if (gateway) {
                shard.fetchedGateway = gateway;
            }
            this.#queue.push(shard);
        }
        await this.handleShards();
        if (this.#queue.length !== 0 && !this.#connectTimeout) {
            this.#connectTimeout = setTimeout(async () => {
                this.#connectTimeout = null;
                await this.connect();
            }, 500).unref();
        }
    }
    disconnect() {
        this.client.emit(Constants_1.Events.Warn, "Disconnecting all Shards");
        for (const shard of this.shards.values()) {
            shard.close(1_000, "Client Disconnect");
        }
    }
    broadcast(payload) {
        for (const shard of this.shards.values()) {
            shard.send(payload);
        }
    }
    setStatus(data) {
        for (const shard of this.shards.keys()) {
            this.setShardStatus(shard, data);
        }
    }
    setShardStatus(shardId, data) {
        const shard = this.shards.get(shardId);
        if (!shard) {
            throw (0, utils_1.MakeError)({
                name: "InvalidShardId",
                message: `The shard with id ${shardId} does not exist`,
            });
        }
        shard.sendPresence(Object.assign({}, shard.presence, data));
    }
    allReady() {
        return [...this.shards.values()].every((shard) => shard.ready && shard.pendingGuilds === 0);
    }
    /**
     * Emits "ready" event to client if all shards has ready
     */
    fireClientReady() {
        // Checking if all shards has ready
        if (this.allReady() &&
            !this.client.isReady &&
            this.client.user &&
            this.client.application) {
            this.client.isReady = true;
            this.client.readyAt = Date.now();
            this.client.emit("ready");
        }
    }
}
exports.WebSocket = WebSocket;
