"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScheduledEvent = exports.Guild = exports.InviteGuild = exports.WelcomeScreen = exports.BaseGuild = void 0;
const ChannelDataManager_1 = require("../manager/ChannelDataManager");
const EmojiDataManager_1 = require("../manager/EmojiDataManager");
const MemberDataManager_1 = require("../manager/MemberDataManager");
const RoleDataManager_1 = require("../manager/RoleDataManager");
const StickerDataManager_1 = require("../manager/StickerDataManager");
const Resolvable_1 = require("../utils/Resolvable");
const v10_1 = require("discord-api-types/v10");
const Base_1 = require("./Base");
const Channel_1 = require("./Channel");
const Member_1 = require("./Member");
const Permission_1 = require("./Permission");
const AuditLog_1 = require("./AuditLog");
const Integration_1 = require("./Integration");
class BaseGuild extends Base_1.Base {
    /**
     * Banner hash
     */
    banner;
    /**
     * Icon hash
     */
    icon;
    /**
     * Enabled guild features
     */
    features;
    /**
     * Splash hash
     */
    splash;
    /**
     * The description of a guild
     */
    description;
    /**
     * Verification level required for the guild
     */
    verificationLevel;
    /**
     * The vanity url code for the guild
     */
    vanityUrlCode;
    /**
     * Guild name (2-100 characters, excluding trailing and leading whitespace)
     */
    name;
    constructor(data) {
        super(data, data.client);
        this._update(data);
    }
    iconURL(format) {
        return (this.icon &&
            v10_1.RouteBases.cdn +
                "/" +
                v10_1.CDNRoutes.guildIcon(this.id, this.icon, format ?? this.icon.startsWith("a_")
                    ? v10_1.ImageFormat.GIF
                    : v10_1.ImageFormat.PNG));
    }
    bannerURL(format) {
        return (this.banner &&
            v10_1.RouteBases.cdn +
                v10_1.CDNRoutes.guildBanner(this.id, this.banner, (format ?? this.icon.startsWith("a_"))
                    ? v10_1.ImageFormat.GIF
                    : v10_1.ImageFormat.PNG));
    }
    _update(data) {
        if ("name" in data)
            this.name = data.name;
        if ("banner" in data && data.banner)
            this.banner = data.banner;
        else
            this.banner ??= null;
        if ("icon" in data)
            this.icon = data.icon;
        else
            this.icon ??= null;
        if ("features" in data && data.features)
            this.features = data.features;
        else
            this.features ??= [];
        if ("splash" in data)
            this.splash = data.splash;
        else
            this.splash ??= null;
        if ("description" in data && data.description)
            this.description = data.description;
        else
            this.description ??= null;
        if ("verification_level" in data && data.verification_level)
            this.verificationLevel = data.verification_level;
        else
            this.verificationLevel ??= null;
        if ("vanity_url_code" in data && data.vanity_url_code)
            this.vanityUrlCode = data.vanity_url_code;
        else
            this.vanityUrlCode ??= null;
        return this;
    }
}
exports.BaseGuild = BaseGuild;
class WelcomeScreen {
    /**
     * The guild for this welcome screen
     */
    guild;
    /**
     * The welcome screen short message
     */
    description;
    /**
     * Suggested channels
     */
    welcomeChannels;
    constructor(data, guild) {
        this.guild = guild;
        this.description = data.description;
        this.welcomeChannels = new Map();
        for (const raw of data.welcome_channels) {
            const welcomeChannel = new Channel_1.WelcomeChannel({
                ...raw,
                client: guild._client,
            });
            this.welcomeChannels.set(welcomeChannel.id, welcomeChannel);
        }
    }
}
exports.WelcomeScreen = WelcomeScreen;
class InviteGuild extends BaseGuild {
    /**
     * Guild NSFW level
     */
    nsfwLevel;
    /**
     * The number of boosts this guild currently has
     */
    premiumSubscriptionCount;
    /**
     * The welcome screen of a Community guild, shown to new members
     */
    welcomeScreen;
    constructor(data) {
        super(data);
        this.nsfwLevel = data.nsfw_level;
        this.premiumSubscriptionCount = data.premium_subscription_count ?? null;
        this.welcomeScreen = data.welcome_screen
            ? new WelcomeScreen(data.welcome_screen, this)
            : null;
    }
}
exports.InviteGuild = InviteGuild;
class Guild extends BaseGuild {
    /**
     * Cache of channels
     */
    channels;
    /**
     * Icon hash, returned when in the template object
     */
    iconHash;
    /**
     * Discovery splash hash; only present for guilds with the "DISCOVERABLE" feature
     */
    discoverySplash;
    /**
     * Id of owner
     */
    ownerId;
    /**
     * Total permissions for the user in the guild (excludes overwrites)
     */
    permissions;
    /**
     * Id of afk channel
     */
    afkChannelId;
    /**
     * Afk timeout in seconds, can be set to: 60, 300, 900, 1800, 3600
     */
    afkTimeout;
    /**
     * True if the server widget is enabled
     */
    widgetEnabled;
    /**
     * The channel id that the widget will generate an invite to, or null if set to no invite
     */
    widgetChannelId;
    /**
     * Default message notifications level
     */
    defaultMessageNotifications;
    /**
     * Explicit content filter level
     */
    explicitContentFilter;
    /**
     * Roles in the guild
     */
    roles;
    /**
     * Custom guild emojis
     */
    emojis;
    /**
     * Required MFA level for the guild
     */
    mfaLevel;
    /**
     * Application id of the guild creator if it is bot-created
     */
    applicationId;
    /**
     * The id of the channel where guild notices such as welcome messages and boost events are posted
     */
    systemChannelId;
    /**
     * System channel flags
     */
    systemChannelFlags;
    /**
     * The id of the channel where Community guilds can display rules and/or guidelines
     */
    rulesChannelId;
    /**
     * The maximum number of presences for the guild (null is always returned, apart from the largest of guilds)
     */
    maxPresences;
    /**
     * The maximum number of members for the guild
     */
    maxMembers;
    /**
     * Premium tier (Server Boost level)
     */
    premiumTier;
    /**
     * The preferred locale of a Community guild; used in server discovery and notices from Discord, and sent in interactions; defaults to "en-US"
     */
    preferredLocale;
    /**
     * The id of the channel where admins and moderators of Community guilds receive notices from Discord
     */
    publicUpdatesChannelId;
    /**
     * The maximum amount of users in a video channel
     */
    maxVideoChannelUsers;
    /**
     * Approximate number of members in this guild, returned from the GET /guilds/<id> endpoint when with_counts is true
     */
    approximateMemberCount;
    /**
     * Approximate number of non-offline members in this guild, returned from the GET /guilds/<id> endpoint when with_counts is true
     */
    approximatePresenceCount;
    /**
     * The welcome screen of a Community guild, shown to new members, returned in an Invite's guild object
     */
    welcomeScreen;
    /**
     * Custom guild stickers
     */
    stickers;
    /**
     * Whether the guild has the boost progress bar enabled
     */
    premiumProgressBarEnabled;
    /**
     * The type of Student Hub the guild is
     */
    hubType;
    /**
     * Members of guild
     */
    members;
    /**
     * Guild NSFW level
     */
    nsfwLevel;
    /**
     * The number of boosts this guild currently has
     */
    premiumSubscriptionCount;
    /**
     * Guild presences
     */
    presences;
    /**
     * Guild shard id
     */
    shardId;
    /**
     * Stage Instances in this guild
     */
    stageInstances;
    /**
     * Voice states in this guild
     */
    voiceStates;
    /**
     * This guild scheduled events
     */
    scheduledEvents;
    /**
     * Guild invites
     */
    invites;
    constructor(data) {
        super(data);
        this.invites = new Map();
        this.voiceStates = new Map();
        this.scheduledEvents = new Map();
        this.stageInstances = new Map();
        this.shardId = "shard_id" in data ? data.shard_id : "0";
        this._update(data);
    }
    /**
     * Returns the client member object
     */
    get clientMember() {
        return this.members.cache.get(this._client.user.id);
    }
    /**
     * Returns a ban object for the given user
     * @param userId
     * @returns
     */
    getMemberBan(userId) {
        return this._client.rest.getGuildBan(this.id, userId);
    }
    getBanList(options) {
        return this._client.rest.getGuildBans(this.id, options);
    }
    /**
     * Create a guild ban, and optionally delete previous messages sent by the banned user.
     * @param userId
     * @param reason
     * @returns
     */
    createMemberBan(userId, options = {}, reason) {
        if (options.deleteMessageDays &&
            !Number.isNaN(options.deleteMessageDays) &&
            !options.deleteMessageSeconds) {
            // Converting days to seconds
            options.deleteMessageSeconds = options.deleteMessageDays * 86400;
        }
        const opts = {
            delete_message_seconds: options.deleteMessageSeconds || 0,
        };
        return this._client.rest.createGuildBan(this.id, userId, opts, reason);
    }
    /**
     * Remove the ban for a user
     * @param userId User id to remove ban
     * @param reason Reason to remove ban
     * @returns
     */
    removeMemberBan(userId, reason) {
        return this._client.rest.removeGuildBan(this.id, userId, reason);
    }
    /**
     * Modify the position of a role in the guild
     * @param roleId Role id to be modified
     * @param newPosition The new position of role
     * @param reason Reason to modify the position
     * @returns
     */
    async setRolePosition(roleId, newPosition, reason) {
        if (Number.isNaN(newPosition)) {
            throw new TypeError("Invalid position");
        }
        const roles = await this._client.rest.modifyRolePosition(this.id, roleId, ~~newPosition, reason);
        const role = roles.find((raw) => raw.id === roleId);
        return this.roles.add(role);
    }
    /**
     * Edit a guild role
     * @param roleId The id of role to be edited
     * @param options The options to edit role
     * @param reason Reason to edit role
     * @returns
     */
    async editRole(roleId, options, reason) {
        const role = await this._client.rest.modifyGuildRole(this.id, roleId, {
            color: options.color,
            hoist: options.hoist,
            icon: options.icon,
            mentionable: options.mentionable,
            name: options.name,
            permissions: options.permissions,
            unicode_emoji: options.unicodeEmoji,
        }, reason);
        const r = this.roles.add(role);
        this._client.roles.cache._add(r);
        return r;
    }
    /**
     * Create a new role for the guild
     * @param options The options to create role
     * @param reason The reason to create role
     * @returns
     */
    async createRole(options, reason) {
        const opts = {};
        opts.color = options.color;
        opts.hoist = options.hoist;
        opts.icon = options.icon;
        opts.mentionable = options.mentionable;
        opts.name = options.name;
        opts.permissions =
            options.permissions instanceof Permission_1.Permissions
                ? options.permissions.allow.toString()
                : options.permissions.toString();
        options.unicodeEmoji;
        const role = await this._client.rest.createGuildRole(this.id, opts, reason);
        const r = this.roles.add(role);
        this._client.roles.cache._add(r);
        return r;
    }
    /**
     * Delete a guild role
     * @param id The id of role to be deleted
     * @param reason Reason to delete role
     * @returns
     */
    deleteRole(id, reason) {
        return this._client.rest.deleteGuildRole(this.id, id, reason);
    }
    /**
     * Begin a prune operation
     * @param options Options to prune members
     * @param reason Reason to prune members
     * @returns
     */
    pruneMembers(options, reason) {
        return this._client.rest.beginGuildPrune(this.id, {
            days: options.days,
            include_roles: options.includeRoles,
            compute_prune_count: options.computePruneCount,
        }, reason);
    }
    /**
     * Remove a member from a guild
     * @param userId The id of the member to be removed
     * @param reason Reason to remove member
     * @returns
     */
    removeMember(userId, reason) {
        return this._client.rest.removeGuildMember(this.id, userId, reason);
    }
    /**
     * Adds a role to a member
     * @param userId Id of the member who will receive the role
     * @param roleId The role to be added to member
     * @param reason Reason to add role
     * @returns
     */
    addMemberRole(userId, roleId, reason) {
        return this._client.rest.addGuildMemberRole(this.id, userId, roleId, reason);
    }
    /**
     * Removes a role from a member
     * @param userId The id of user to remove role
     * @param roleId The id of role to be removed from member
     * @param reason The reason to remove role
     * @returns
     */
    removeMemberRole(userId, roleId, reason) {
        return this._client.rest.removeGuildMemberRole(this.id, userId, roleId, reason);
    }
    async editMember(userId, options, reason) {
        const opts = {};
        if (userId !== "@me") {
            opts.channel_id = options.channelId;
            opts.communication_disabled_until = options.communicationDisabledUntil;
            opts.deaf = options.deaf;
            opts.mute = options.mute;
            opts.roles = options.roles;
        }
        opts.nick = options.nick;
        const member = await this._client.rest.modifyGuildMember(this.id, userId, opts, reason);
        return this.members.add(new Member_1.Member(member, this));
    }
    async getAuditLogs() {
        const data = await this._client.rest.getGuildAuditLog(this.id);
        return new AuditLog_1.AuditLog({ ...data, client: this._client, guild: this });
    }
    getInvites() {
        return this._client.rest.getGuildInvites(this.id);
    }
    async getIntegrations() {
        const rawIntegrations = await this._client.rest.getGuildIntegrations(this.id);
        const integrations = [];
        for (const rawIntegration of rawIntegrations) {
            integrations.push(new Integration_1.Integration(rawIntegration, this));
        }
        return integrations;
    }
    deleteIntegration(id) {
        return this._client.rest.deleteGuildIntegration(this.id, id);
    }
    createScheduledEvent(options, reason) {
        return this._client.rest.createGuildScheduledEvent(this.id, {
            channel_id: options.channelId,
            description: options.description,
            entity_metadata: options.entityMetadata,
            entity_type: options.entityType,
            image: options.image,
            name: options.name,
            scheduled_start_time: options.scheduledStartTime,
            scheduled_end_time: options.scheduledEndTime,
            privacy_level: options.privacyLevel,
        }, reason);
    }
    createAutoModerationRule(options, reason) {
        return this._client.rest.createGuildAutoModerationRule(this.id, {
            actions: options.actions,
            enabled: options.enabled,
            event_type: options.eventType,
            exempt_channels: options.exemptChannels,
            exempt_roles: options.exemptRoles,
            name: options.name,
            trigger_type: options.triggerType,
            trigger_metadata: options.triggerMetadata && {
                allow_list: options.triggerMetadata.allowList,
                keyword_filter: options.triggerMetadata.keywordFilter,
                mention_total_limit: options.triggerMetadata.mentionTotalLimit,
                presets: options.triggerMetadata.presets,
                regex_patterns: options.triggerMetadata.regexPatterns,
            },
        }, reason);
    }
    deleteAutoModerationRule(autoModerationRuleId, reason) {
        return this._client.rest.deleteGuildAutoModerationRule(this.id, autoModerationRuleId, reason);
    }
    editAutoModerationRule(autoModerationRuleId, options, reason) {
        return this._client.rest.modifyGuildModerationRule(this.id, autoModerationRuleId, {
            actions: options.actions,
            enabled: options.enabled,
            event_type: options.eventType,
            exempt_channels: options.exemptChannels,
            exempt_roles: options.exemptRoles,
            name: options.name,
            trigger_metadata: options.triggerMetadata && {
                allow_list: options.triggerMetadata.allowList,
                keyword_filter: options.triggerMetadata.keywordFilter,
                mention_total_limit: options.triggerMetadata.mentionTotalLimit,
                presets: options.triggerMetadata.presets,
                regex_patterns: options.triggerMetadata.regexPatterns,
            },
        }, reason);
    }
    /**
     * Create a channel in this guild
     * @param options The options to create channel
     * @param reason Reason to create channel
     * @returns
     */
    async createChannel(options, reason) {
        const opts = {
            position: options.position,
            name: options.name,
            type: options.type,
            permission_overwrites: options.permissionOverwrites,
            nsfw: options.nsfw,
            parent_id: options.parentId,
            flags: options.flags,
            topic: options.topic,
            bitrate: options.bitrate,
            video_quality_mode: options.videoQualityMode,
            default_auto_archive_duration: options.defaultAutoArchiveDuration,
            default_forum_layout: options.defaultForumLayout,
            default_sort_order: options.defaultSortOrder,
            default_reaction_emoji: options.defaultReactionEmoji,
            rtc_region: options.rtcRegion,
            user_limit: options.userLimit,
            rate_limit_per_user: options.rateLimitPerUser,
        };
        const data = await this._client.rest.createGuildChannel(this.id, opts, reason);
        return Resolvable_1.Resolvable.resolveChannel(Channel_1.Channel.from({ ...data, client: this._client }, this), this._client, this);
    }
    /**
     * Delete a channel in this guild
     * @param id The id of channel to be deleted
     * @param reason Reason to delete channel
     * @returns
     */
    deleteChannel(id, reason) {
        return this._client.rest.deleteChannel(id, reason);
    }
    /**
     * Get permissions of user
     * @param userId user id to get permissions
     * @returns
     */
    permissionsOf(userId) {
        let member;
        if (userId instanceof Member_1.Member) {
            member = userId;
            userId = member.id;
        }
        if (userId === this.ownerId)
            return new Permission_1.Permissions(Permission_1.Permissions.All);
        const raw = this.roles.cache.get(this.id)?.permissions;
        let perms = typeof raw === "string" ? BigInt(raw) : raw?.allow || 0n;
        if (perms & Permission_1.Permissions.Flags.Administrator)
            return new Permission_1.Permissions(Permission_1.Permissions.All);
        if (!member)
            member = this.members.cache.get(userId);
        for (const role of member.roles) {
            if (role) {
                const perm = typeof role.permissions === "string"
                    ? BigInt(role.permissions)
                    : role.permissions.allow;
                if (perm & Permission_1.Permissions.Flags.Administrator)
                    perms = Permission_1.Permissions.All;
                else
                    perms |= perm;
            }
        }
        return new Permission_1.Permissions(perms);
    }
    /**
     * Create a new guild command
     * @param options Options to create command
     * @returns
     */
    createApplicationCommand(options) {
        return this._client.application.createGuildCommand(this.id, options);
    }
    /**
     * Edit a guild command
     * @param commandId The id of command to be edited
     * @param options Options to edit command
     * @returns
     */
    editApplicationCommand(commandId, options) {
        return this._client.application.editGuildCommand(this.id, commandId, options);
    }
    /**
     * Delete a guild command
     * @param commandId The id of command to be deleted
     * @returns
     */
    deleteApplicationCommand(commandId) {
        return this._client.application.deleteGuildCommand(this.id, commandId);
    }
    /**
     * Takes a list of application commands, overwriting the existing command list for this application for the targeted guild
     * @param commands The array of commands
     * @returns
     */
    bulkOverwriteApplicationCommands(commands) {
        return this._client.application.bulkOverwriteGuildCommands(this.id, commands);
    }
    /**
     * Create a new emoji for the guild. Requires the `CreateGuildExpressions` permission.
     *
     * Returns the new emoji object on success. Fires a `guildEmojisUpdate` Gateway event.
     * @param options The options to create emoji
     * @param reason Reason to create emoji
     */
    async createEmoji(options, reason) {
        let emojiImage;
        if (options.data && options.data instanceof Buffer) {
            emojiImage = `data:image/jpg;base64,${options.data.toString("base64")}`;
        }
        else if (options.data &&
            typeof options.data === "string" &&
            options.data.startsWith("data:")) {
            emojiImage = options.data;
        }
        else {
            throw new TypeError("Invalid emoji data");
        }
        const data = await this._client.rest.createEmoji(this.id, {
            name: options.name,
            image: emojiImage,
            roles: options.roles,
        }, reason);
        return this.emojis.add(data);
    }
    /**
     * Modify the given emoji.
     *
     * For emojis created by the current user, requires either the `createGuildExpressions` or `manageGuildExpressions` permission.
     *
     * For other emojis, requires the `manageGuildExpressions` permission.
     *
     * Returns the updated emoji object on success. Fires a `guildEmojisUpdate` Gateway event.
     * @param emojiId The id of emoji to be edited
     * @param options The options to edit emoji
     * @param reason Reason to edit emoji
     * @returns
     */
    async editEmoji(emojiId, options, reason) {
        const data = await this._client.rest.modifyEmoji(this.id, emojiId, options, reason);
        return this.emojis.add(data);
    }
    /**
     * Delete the given emoji.
     *
     * For emojis created by the current user, requires either the `createGuildExpressions` or `manageGuildExpressions`S permission.
     *
     * For other emojis, requires the `manageGuildExpressions` permission.
     * @param emojiId The id of emoji to be deleted
     * @param reason Reason to delete emoji
     * @returns
     */
    deleteEmoji(emojiId, reason) {
        return this._client.rest.deleteEmoji(this.id, emojiId, reason);
    }
    /**
     * The acronym that shows up in place of a guild icon
     * @returns
     */
    get nameAcronym() {
        return this.name
            .replace(/'s /g, " ")
            .replace(/\w+/g, (e) => e[0])
            .replace(/\s/g, "");
    }
    /**
     * Edit the guild
     * @param options Options to edit guild
     * @param reason Reason to edit
     * @returns
     */
    async edit(options, reason) {
        const data = await this._client.rest.modifyGuild(this.id, {
            afk_channel_id: options.afkChannelId,
            afk_timeout: options.afkTimeout,
            banner: options.banner,
            default_message_notifications: options.defaultMessageNotifications,
            description: options.description,
            discovery_splash: options.discoverySplash,
            explicit_content_filter: options.explicitContentFilter,
            features: options.features,
            icon: options.icon,
            name: options.name,
            owner_id: options.ownerId,
            preferred_locale: options.preferredLocale,
            premium_progress_bar_enabled: options.premiumProgressBarEnabled,
            public_updates_channel_id: options.publicUpdatesChannelId,
            region: options.region,
            rules_channel_id: options.rulesChannelId,
            safety_alerts_channel_id: options.safetyAlertsChannelId,
            splash: options.splash,
            system_channel_flags: options.systemChannelFlags,
            system_channel_id: options.systemChannelId,
            verification_level: options.verificationLevel,
        }, reason);
        return this._update(data);
    }
    _update(data) {
        if ("nsfw_level" in data)
            this.nsfwLevel = data.nsfw_level;
        if ("premium_subscription_count" in data && data.premium_subscription_count)
            this.premiumSubscriptionCount = data.premium_subscription_count;
        else
            this.premiumSubscriptionCount ??= 0;
        if ("icon_hash" in data && data.icon_hash)
            this.iconHash = data.icon_hash;
        else
            this.iconHash ??= null;
        if ("widget_channel_id" in data)
            this.widgetChannelId = data.widget_channel_id;
        if ("widget_enabled" in data)
            this.widgetEnabled = data.widget_enabled;
        if ("owner_id" in data)
            this.ownerId = data.owner_id;
        if ("discovery_splash" in data)
            this.discoverySplash = data.discovery_splash;
        if ("afk_channel_id" in data)
            this.afkChannelId = data.afk_channel_id;
        if ("afk_timeout" in data)
            this.afkTimeout = data.afk_timeout;
        if ("public_updates_channel_id" in data)
            this.publicUpdatesChannelId = data.public_updates_channel_id;
        else
            this.publicUpdatesChannelId ??= null;
        if ("max_members" in data)
            this.maxMembers = data.max_members;
        if ("approximate_member_count" in data)
            this.approximateMemberCount = data.approximate_member_count;
        if ("approximate_presence_count" in data)
            this.approximatePresenceCount = data.approximate_presence_count;
        if ("permissions" in data && data.permissions)
            this.permissions = new Permission_1.Permissions(BigInt(data.permissions));
        else
            this.permissions ??= new Permission_1.Permissions(0n);
        this.channels ??= new ChannelDataManager_1.GuildChannelDataManager(this._client.cache._cacheLimit("channels"), this._client.cache, this);
        this.roles ??= new RoleDataManager_1.RoleDataManager(this._client.cache._cacheLimit("roles"), this._client.cache, this);
        this.emojis ??= new EmojiDataManager_1.GuildEmojiDataManager(this._client.cache._cacheLimit("emojis"), this._client.cache, this);
        this.stickers ??= new StickerDataManager_1.GuildStickerDataManager(this._client.cache._cacheLimit("stickers"), this._client.cache, this);
        this.members ??= new MemberDataManager_1.MemberDataManager(this._client.cache._cacheLimit("members"), this._client.cache, this);
        if ("roles" in data && Array.isArray(data.roles)) {
            for (const role of data.roles) {
                const r = this.roles.add(role);
                this._client.roles.cache._add(r);
            }
        }
        if ("emojis" in data && Array.isArray(data.emojis)) {
            for (const emoji of data.emojis) {
                this.emojis.add(emoji);
                this._client.cache.emojis.add(emoji);
            }
        }
        if ("stickers" in data && Array.isArray(data.stickers)) {
            for (const sticker of data.stickers) {
                this.stickers.add(sticker);
            }
        }
        super._update(data);
        return this;
    }
    /**
     * Update information of this guild
     *
     * Util if this is forged
     * @returns
     */
    async fetchInformation() {
        const data = await this._client.rest.getGuild(this.id);
        return this._update(data);
    }
}
exports.Guild = Guild;
class ScheduledEvent extends Base_1.Base {
    /**
     * The channel id in which the scheduled event will be hosted, or null if entity type is EXTERNAL The channel id in which the scheduled event will be hosted, or null if entity type is EXTERNAL The channel id in which the scheduled event will be hosted, or null if entity type is EXTERNAL
     */
    channelId;
    /**
     * The user that created the scheduled event
     */
    creator;
    /**
     * The id of the user that created the scheduled event
     */
    creatorId;
    /**
     * The description of the scheduled event
     */
    description;
    /**
     * The id of the hosting entity associated with the scheduled event
     */
    entityId;
    /**
     * The type of hosting entity associated with the scheduled event
     */
    entityType;
    /**
     * The entity metadata for the scheduled event The entity metadata for the scheduled event The entity metadata for the scheduled event
     */
    entityMetadata;
    /**
     * The privacy level of the scheduled event
     */
    privacyLevel;
    /**
     * The number of users subscribed to the scheduled event
     */
    userCount;
    /**
     * The status of the scheduled event
     */
    status;
    /**
     * The guild of scheduled event
     */
    guild;
    constructor(data, guild) {
        super(data, guild._client);
        this.guild = guild;
        this.channelId = data.channel_id;
        this.creator = data.creator
            ? this._client.cache.users.add(data.creator)
            : null;
        this.creatorId = data.creator_id;
        this.description = data.description;
        this.entityId = data.entity_id;
        this.entityType = data.entity_type;
        this.entityMetadata = data.entity_metadata;
        this.privacyLevel = data.privacy_level;
        this.userCount = data.user_count;
        this.status = data.status;
    }
}
exports.ScheduledEvent = ScheduledEvent;
