"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WelcomeChannel =
  exports.StageChannel =
  exports.VoiceChannel =
  exports.BaseVoiceChannel =
  exports.ForumChannel =
  exports.CategoryChannel =
  exports.DMChannel =
  exports.ThreadChannel =
  exports.GuildTextChannel =
  exports.GuildChannel =
  exports.TextBasedChannel =
  exports.Channel =
  exports.ChannelFlags =
    void 0;
const ChannelDataManager_1 = require("../../manager/ChannelDataManager");
const DataManager_1 = require("../../manager/DataManager");
const MemberDataManager_1 = require("../../manager/MemberDataManager");
const MessageDataManager_1 = require("../../manager/MessageDataManager");
const Constants_1 = require("../../utils/Constants");
const Resolvable_1 = require("../../utils/Resolvable");
const index_1 = require("../../utils/index");
const v10_1 = require("discord-api-types/v10");
const ts_mixer_1 = require("ts-mixer");
const Base_1 = require("./Base");
const BitField_1 = require("./BitField");
const Emoji_1 = require("./Emoji");
const Member_1 = require("./Member");
const Message_1 = require("./Message");
const Permission_1 = require("./Permission");
const Webhook_1 = require("./Webhook");
class ChannelFlags extends BitField_1.BitField {
  constructor(flags) {
    super(flags, v10_1.ChannelFlags);
  }
  static Flags = v10_1.ChannelFlags;
}
exports.ChannelFlags = ChannelFlags;
class Channel extends Base_1.Base {
  /**
   * The type of the channel
   *
   * @See https://discord.com/developers/docs/resources/channel#channel-object-channel-types
   */
  type;
  /**
   * The name of the channel (2-100 characters)
   */
  name;
  /**
   * The flags of the channel
   */
  flags;
  constructor(data) {
    super(data, data.client);
    this.type = data.type;
    this.name = typeof data.name === "string" ? data.name : null;
    this.flags = data.flags ? new ChannelFlags(data.flags) : null;
  }
  async edit(options, reason) {
    const data = await this._client.rest.modifyChannel(
      this.id,
      {
        name: options.name,
        locked: options.locked,
        type: options.type,
        invitable: options.invitable,
        default_auto_archive_duration: options.defaultAutoArchiveDuration,
        default_reaction_emoji: options.defaultReactionEmoji,
        default_sort_order: options.defaultSortOrder,
        default_thread_rate_limit_per_user:
          options.defaultThreadRateLimitPerUser,
        default_forum_layout: options.defaultForumLayout,
        topic: options.topic,
        bitrate: options.bitrate,
        available_tags: options.availableTags,
        auto_archive_duration: options.autoArchiveDuration,
        video_quality_mode: options.videoQualityMode,
        rtc_region: options.rtcRegion,
        user_limit: options.userLimit,
        position: options.position,
        permission_overwrites: options.permissionOverwrites,
        applied_tags: options.appliedTags,
        archived: options.archived,
        parent_id: options.parentId,
        rate_limit_per_user: options.rateLimitPerUser,
        flags: options.flags,
      },
      reason,
    );
    return this._update(data);
  }
  toString() {
    return (0, Constants_1.channelMention)(this.id);
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "type",
      "name",
      "flags",
      "rawData",
      "id",
      "createdAt",
    ]);
  }
  _update(data) {
    if ("name" in data) this.name = data.name;
    if ("flags" in data)
      this.flags = (data.flags && new ChannelFlags(data.flags)) ?? null;
    return this;
  }
  isDM() {
    return this instanceof DMChannel;
  }
  isText() {
    return TextBasedChannel.isBased(this);
  }
  isGuildText() {
    return this instanceof GuildTextChannel;
  }
  isGuildChannel() {
    return (
      this instanceof GuildChannel ||
      this.isGuildText() ||
      this.isVoice() ||
      this.isThread() ||
      this.isStage()
    );
  }
  isThread() {
    return this instanceof ThreadChannel;
  }
  isStage() {
    return this instanceof StageChannel;
  }
  isVoice() {
    return this instanceof VoiceChannel;
  }
  get forged() {
    return Boolean(this.id && !this.flags && !this.name);
  }
  /**
   * Update information of this channel
   *
   * Util if this is forged
   * @returns
   */
  async fetchInformation() {
    const data = await this._client.rest.getChannel(this.id);
    return this._update(data);
  }
  static from(data, guild) {
    switch (data.type) {
      case v10_1.ChannelType.GuildText: {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        if (!guild) return new Channel(data);
        return new GuildTextChannel(data, guild);
      }
      case v10_1.ChannelType.DM: {
        return new DMChannel(data);
      }
      case v10_1.ChannelType.GuildCategory: {
        if (!guild) return new Channel(data);
        return new CategoryChannel(data, guild);
      }
      case v10_1.ChannelType.GuildForum: {
        if (!guild) return new Channel(data);
        return new ForumChannel(data, guild);
      }
      case v10_1.ChannelType.GuildVoice: {
        if (!guild) return new Channel(data);
        return new VoiceChannel(data, guild);
      }
      case v10_1.ChannelType.GuildStageVoice: {
        if (!guild) return new Channel(data);
        return new StageChannel(data, guild);
      }
      case v10_1.ChannelType.PrivateThread:
      case v10_1.ChannelType.PublicThread: {
        if (!guild) return new Channel(data);
        return new ThreadChannel(data, guild);
      }
      default: {
        return new Channel(data);
      }
    }
  }
}
exports.Channel = Channel;
class TextBasedChannel extends Channel {
  /**
   * The id of the last message sent in this channel (may not point to an existing or valid message)
   */
  lastMessageId;
  /**
   * When the last pinned message was pinned.
   */
  lastPinTimestamp;
  /**
   * Channel messages
   */
  messages;
  constructor(data) {
    super(data);
    this.messages = new MessageDataManager_1.MessageDataManager(
      this._client.options.cache?.messageCacheLimitPerChannel || Infinity,
      this._client.cache,
      this,
    );
    this.lastMessageId = data.last_message_id;
    this.lastPinTimestamp = data.last_pin_timestamp
      ? Date.parse(data.last_pin_timestamp)
      : null;
  }
  async createMessage(content) {
    const message = await this._client.rest.createMessage(
      this.id,
      (0, index_1.transformMessagePostData)(content),
    );
    return Resolvable_1.Resolvable.resolveMessage(
      new Message_1.Message({
        client: this._client,
        ...message,
      }),
      this._client,
    );
  }
  deleteMessage(id, reason) {
    return this._client.rest.deleteMessage(this.id, id, reason);
  }
  async bulkDeleteMessages(messages) {
    if (typeof messages === "number") {
      messages = await this.messages
        .fetch({
          limit: messages,
        })
        .then((messageArr) => messageArr.map((message) => message.id));
    }
    // Resolving message array
    messages = messages.map((message) =>
      typeof message === "string" ? message : message.id,
    );
    return this._client.rest.bulkDeleteMessages(this.id, messages);
  }
  sendTyping() {
    return this._client.rest.triggerTyping(this.id);
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "id",
      "messages",
      "lastMessageId",
      "lastPinTimestamp",
      "name",
      "type",
      "rawData",
    ]);
  }
  static isBased(channel) {
    return (
      channel instanceof TextBasedChannel ||
      channel instanceof GuildTextChannel ||
      channel instanceof StageChannel ||
      channel instanceof VoiceChannel ||
      channel instanceof DMChannel ||
      channel instanceof ThreadChannel
    );
  }
}
exports.TextBasedChannel = TextBasedChannel;
class GuildChannel extends Channel {
  guild;
  /**
   * The id of the guild (may be missing for some channel objects received over gateway guild dispatches)
   */
  guildId;
  /**
   * ID of the parent category for a channel (each parent category can contain up to 50 channels)
   * OR
   * ID of the parent channel for thread
   */
  parentId;
  /**
   * Explicit permission overwrites for members and roles
   */
  permissionOverwrites;
  /**
   * Sorting position of the channel
   */
  position;
  /**
   * Whether the channel is nsfw
   */
  nsfw;
  /**
   * The threads in this channel
   */
  threads;
  constructor(data, guild) {
    super(data);
    this.guild = guild;
    this.guildId = data.guild_id;
    this.permissionOverwrites = new DataManager_1.DataCache();
    this.threads = new DataManager_1.DataCache(
      this._client.cache._cacheLimit("threads"),
    );
    this._update(data);
    if (data.permission_overwrites) {
      for (const perm of data.permission_overwrites) {
        const perms = new Permission_1.PermissionOverwrite(
          {
            ...perm,
            client: this._client,
          },
          this,
        );
        this.permissionOverwrites._add(perms);
      }
    }
  }
  /**
   *
   * @param overwriteId User or Role id
   * @param options Options to edit permissions
   */
  editPermissions(overwriteId, options, reason) {
    return this._client.rest.editChannelPermissions(
      this.id,
      overwriteId,
      options,
      reason,
    );
  }
  _update(data) {
    if ("position" in data) this.position = data.position;
    if ("nsfw" in data) this.nsfw = Boolean(data.nsfw);
    if ("parent_id" in data) this.parentId = data.parent_id;
    super._update(data);
    return this;
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "name",
      "nsfw",
      "parentId",
      "permissionOverwrites",
      "position",
      "rawData",
      "threads",
      "type",
    ]);
  }
}
exports.GuildChannel = GuildChannel;
class GuildTextChannel extends (0, ts_mixer_1.Mixin)(
  GuildChannel,
  TextBasedChannel,
) {
  /**
   * The channel topic (0-4096 characters for forum channels, 0-1024 characters for all others)
   */
  topic;
  /**
   * Amount of seconds a user has to wait before sending another message (0-21600)
   */
  rateLimitPerUser;
  constructor(data, guild) {
    super(data, guild);
    this._update(data);
    this.messages = new MessageDataManager_1.MessageDataManager(
      this._client.options.cache?.messageCacheLimitPerChannel || Infinity,
      this._client.cache,
      this,
    );
  }
  // Override TextBasedChannel.createMessage for guild
  async createMessage(content) {
    const message = await this._client.rest.createMessage(
      this.id,
      (0, index_1.transformMessagePostData)(content),
    );
    return Resolvable_1.Resolvable.resolveMessage(
      new Message_1.Message(
        {
          client: this._client,
          ...message,
        },
        this.guild,
      ),
      this._client,
    );
  }
  async createWebhook(data) {
    const rawWebhook = await this._client.rest.createWebhook(this.id, data);
    const webhook = new Webhook_1.Webhook({
      ...rawWebhook,
      client: this._client,
    });
    return webhook;
  }
  async fetchWebhook(webhookId) {
    const data = await this._client.rest.getWebhook(webhookId);
    return new Webhook_1.Webhook({ ...data, client: this._client });
  }
  async fetchWebhooks() {
    const data = await this._client.rest.getChannelWebhooks(this.id);
    return data.map(
      (d) => new Webhook_1.Webhook({ ...d, client: this._client }),
    );
  }
  _update(data) {
    if ("topic" in data) this.topic = data.topic;
    if ("rate_limit_per_user" in data)
      this.rateLimitPerUser = data.rate_limit_per_user;
    super._update(data);
    return this;
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "lastMessageId",
      "lastPinTimestamp",
      "messages",
      "name",
      "nsfw",
      "parentId",
      "permissionOverwrites",
      "position",
      "rateLimitPerUser",
      "rawData",
      "threads",
      "topic",
      "type",
    ]);
  }
}
exports.GuildTextChannel = GuildTextChannel;
class ThreadChannel extends TextBasedChannel {
  /**
   * The client users member for the thread
   */
  member;
  /**
   * Number of messages (not including the initial message or deleted messages) in a thread
   *
   * If the thread was created before July 1, 2022, it stops counting at 50 messages
   */
  messageCount;
  /**
   * The approximate member count of the thread, does not count above 50 even if there are more members
   */
  memberCount;
  /**
   * Id of the thread creato
   */
  ownerId;
  /**
   * Number of messages ever sent in a thread
   */
  totalMessageSent;
  /**
   * The Ids of the set of tags that have been applied to a thread in a forum channel
   */
  appliedTags;
  /**
   * The channel associated with this thread
   */
  channel;
  /**
   * The id of the guild
   */
  guildId;
  /**
   * The thread guild
   */
  guild;
  /**
   * Whether the thread is archived
   */
  archived;
  /**
   * Timestamp when the thread's archive status was last changed, used for calculating recent activity
   */
  archiveTimestamp;
  /**
   * Duration in minutes to automatically archive the thread after recent activity, can be set to: 60, 1440, 4320, 10080
   */
  autoArchiveDuration;
  /**
   * Timestamp when the thread was created; only populated for threads created after 2022-01-09
   */
  createTimestamp;
  /**
   * Whether non-moderators can add other non-moderators to the thread; only available on private threads
   */
  invitable;
  /**
   * Whether the thread is locked
   */
  locked;
  /**
   * Members in this thread
   */
  members;
  constructor(data, guild) {
    super(data);
    this.guild = guild;
    this.ownerId = data.owner_id;
    this.guildId = data.guild_id || guild.id;
    this.channel = data.parent_id
      ? guild.channels.cache.get(data.parent_id)
      : null;
    this.member = data.member
      ? new Member_1.ThreadMember(
          { ...data.member, client: this._client },
          this,
        )
      : null;
    this.members = new DataManager_1.DataCache();
    this._update(data);
  }
  addMember(userId) {
    return this._client.rest.addThreadMember(this.id, userId);
  }
  removeMember(userId) {
    return this._client.rest.removeThreadMember(this.id, userId);
  }
  async getMember(userId) {
    const member = await this._client.rest.getThreadMember(this.id, userId);
    return new Member_1.ThreadMember({ ...member, client: this._client }, this);
  }
  /**
   * Removes the current user from a thread
   */
  leave() {
    return this._client.rest.leaveThread(this.id);
  }
  _update(data) {
    if ("applied_tags" in data) this.appliedTags = data.applied_tags;
    if ("total_message_sent" in data)
      this.totalMessageSent = data.total_message_sent;
    if ("message_count" in data) this.messageCount = data.message_count ?? 0;
    if ("member_count" in data) this.memberCount = data.member_count;
    // MetaData
    if ("thread_metadata" in data && data.thread_metadata) {
      const metadata = data.thread_metadata;
      if ("archived" in metadata) this.archived = metadata.archived;
      if ("archive_timestamp" in metadata)
        this.archiveTimestamp = Date.parse(metadata.archive_timestamp);
      if ("auto_archive_duration" in metadata)
        this.autoArchiveDuration = metadata.auto_archive_duration;
      if ("create_timestamp" in metadata)
        this.createTimestamp = metadata.create_timestamp
          ? Date.parse(metadata.create_timestamp)
          : null;
      if ("invitable" in metadata) this.invitable = Boolean(metadata.invitable);
      if ("locked" in metadata) this.locked = Boolean(metadata.locked);
    }
    super._update(data);
    return this;
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "appliedTags",
      "archiveTimestamp",
      "archived",
      "autoArchiveDuration",
      "channel",
      "createTimestamp",
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "invitable",
      "autoArchiveDuration",
      "leave",
      "locked",
      "member",
      "memberCount",
      "members",
      "messages",
      "name",
      "ownerId",
      "rawData",
      "totalMessageSent",
      "type",
    ]);
  }
}
exports.ThreadChannel = ThreadChannel;
class DMChannel extends TextBasedChannel {
  /**
   * The user of the dm
   */
  user;
  /**
   * The user id of the dm
   */
  userId;
  constructor(data) {
    super({ ...data, client: data.client });
    this.user = this._client.cache.users.add(data.recipients?.[0]);
    this.userId = this.user.id;
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "id",
      "messages",
      "lastMessageId",
      "lastPinTimestamp",
      "name",
      "type",
      "rawData",
      "user",
      "userId",
    ]);
  }
}
exports.DMChannel = DMChannel;
class CategoryChannel extends GuildChannel {
  /**
   * The channels present in this category
   */
  channels;
  constructor(data, guild) {
    super(data, guild);
    this.channels = new ChannelDataManager_1.GuildChannelDataManager(
      this._client.cache._cacheLimit("channels"),
      this._client.cache,
      this.guild,
    );
    for (const [, channel] of this.guild.channels.cache.filter((ch) => {
      const parentId = ch.parentId || ch.parent_id;
      return parentId === this.id;
    })) {
      this.channels.add(channel);
    }
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "name",
      "nsfw",
      "parentId",
      "permissionOverwrites",
      "position",
      "rawData",
      "threads",
      "type",
      "channels",
    ]);
  }
}
exports.CategoryChannel = CategoryChannel;
class ForumChannel extends GuildChannel {
  /**
   * Default duration for newly created threads, in minutes, to automatically archive the thread after recent activity
   */
  defaultAutoArchiveDuration;
  /**
   * The default layout type used to display posts in a forum channel. Defaults to 0, which indicates a layout view has not been set by a channel admin
   */
  defaultForumLayout;
  /**
   * The emoji to show in the add reaction button on a thread in a forum channel
   */
  defaultReactionEmoji;
  /**
   * The default sort order type used to order posts in a forum channel
   */
  defaultSortOrder;
  /**
   * The initial rate_limit_per_user to set on newly created threads. This field is copied to the thread at creation time and does not live update
   */
  defaultThreadRateLimitPerUser;
  /**
   * The set of tags that can be used in a forum channel
   */
  availableTags;
  constructor(data, guild) {
    super(data, guild);
    this._update(data);
    this.threads = new DataManager_1.DataCache(
      this._client.cache._cacheLimit("threads"),
    );
  }
  _update(data) {
    if ("available_tags" in data) this.availableTags = data.available_tags;
    if ("default_auto_archive_duration" in data)
      this.defaultAutoArchiveDuration = data.default_auto_archive_duration;
    if ("default_forum_layout" in data)
      this.defaultForumLayout = data.default_forum_layout;
    if ("default_sort_order" in data)
      this.defaultSortOrder = data.default_sort_order;
    if ("default_thread_rate_limit_per_user" in data)
      this.defaultThreadRateLimitPerUser =
        data.default_thread_rate_limit_per_user;
    if ("default_reaction_emoji" in data && data.default_reaction_emoji) {
      this.defaultReactionEmoji = {
        emojiId: data.default_reaction_emoji.emoji_id,
        emojiName: data.default_reaction_emoji.emoji_name,
      };
    }
    super._update(data);
    return this;
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "name",
      "nsfw",
      "parentId",
      "permissionOverwrites",
      "position",
      "rawData",
      "threads",
      "type",
      "defaultAutoArchiveDuration",
      "defaultForumLayout",
      "defaultReactionEmoji",
      "defaultSortOrder",
      "defaultThreadRateLimitPerUser",
      "availableTags",
      "threads",
    ]);
  }
}
exports.ForumChannel = ForumChannel;
class BaseVoiceChannel extends GuildChannel {
  /**
   * Voice region id for the voice or stage channel
   */
  rtcRegion;
  /**
   * The bitrate (in bits) of the voice channel
   */
  bitrate;
  /**
   * The user limit of the voice channel
   */
  userLimit;
  /**
   * Members in this voice channel
   */
  members;
  constructor(data, guild) {
    super(data, guild);
    this.guild = guild;
    this.guildId = guild.id;
    this._update(data);
    this.members = new MemberDataManager_1.MemberDataManager(
      Infinity,
      this._client.cache,
      guild,
    );
  }
  _update(data) {
    if ("bitrate" in data && data.bitrate) this.bitrate = data.bitrate;
    if ("rtc_region" in data && data.rtc_region)
      this.rtcRegion = data.rtc_region;
    if ("user_limit" in data && data.user_limit)
      this.userLimit = data.user_limit;
    return this;
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "name",
      "nsfw",
      "parentId",
      "permissionOverwrites",
      "position",
      "rawData",
      "threads",
      "type",
      "bitrate",
      "userLimit",
      "rtcRegion",
      "members",
    ]);
  }
}
exports.BaseVoiceChannel = BaseVoiceChannel;
class VoiceChannel extends (0, ts_mixer_1.Mixin)(
  BaseVoiceChannel,
  TextBasedChannel,
) {
  /**
   * The camera video quality mode of the voice channel
   */
  videoQualityMode;
  constructor(data, guild) {
    super(data, guild);
    this.videoQualityMode = data.video_quality_mode ?? null;
    this.messages = new MessageDataManager_1.MessageDataManager(
      this._client.options.cache?.messageCacheLimitPerChannel || Infinity,
      this._client.cache,
      this,
    );
  }
  // Override TextBasedChannel.createMessage for guild
  async createMessage(content) {
    const message = await this._client.rest.createMessage(
      this.id,
      (0, index_1.transformMessagePostData)(content),
    );
    return Resolvable_1.Resolvable.resolveMessage(
      new Message_1.Message(
        {
          client: this._client,
          ...message,
        },
        this.guild,
      ),
      this._client,
    );
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "createdAt",
      "flags",
      "guild",
      "guildId",
      "id",
      "name",
      "nsfw",
      "parentId",
      "permissionOverwrites",
      "position",
      "rawData",
      "threads",
      "type",
      "bitrate",
      "userLimit",
      "rtcRegion",
      "members",
      "messages",
      "videoQualityMode",
    ]);
  }
}
exports.VoiceChannel = VoiceChannel;
class StageChannel extends (0, ts_mixer_1.Mixin)(
  BaseVoiceChannel,
  TextBasedChannel,
) {
  constructor(data, guild) {
    super(data, guild);
  }
  get stageInstance() {
    for (const instance of this.guild.stageInstances.values()) {
      if (instance.channel_id === this.id) return instance;
    }
    return null;
  }
  async createStageInstance(options) {
    const instance = await this._client.rest.createStageInstance({
      channel_id: options.channelId,
      privacy_level: options.privacyLevel,
      send_start_notification: options.sendStartNotification,
      topic: options.topic,
    });
    this.guild.stageInstances.set(instance.id, instance);
    return instance;
  }
}
exports.StageChannel = StageChannel;
class WelcomeChannel extends Base_1.Base {
  /**
   * The channel id that is suggested
   */
  channelId;
  /**
   * The description shown for the channel
   */
  description;
  /**
   * The emoji id of the emoji that is shown on the left of the channel
   */
  emojiId;
  /**
   * The emoji name of the emoji that is shown on the left of the channel
   */
  emojiName;
  constructor(data) {
    super(data, data.client, data.channel_id);
    this.channelId = data.channel_id;
    this.description = data.description;
    this.emojiId = data.emoji_id;
    this.emojiName = data.emoji_name;
  }
  get channel() {
    return this._client.channels.cache.get(this.channelId);
  }
  get emoji() {
    return (
      // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
      this._client.cache.emojis.cache.get(this.emojiId) ??
      new Emoji_1.Emoji({
        id: this.emojiId,
        name: this.emojiName,
      })
    );
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "channel",
      "channelId",
      "emoji",
      "emojiId",
      "description",
      "emojiName",
    ]);
  }
}
exports.WelcomeChannel = WelcomeChannel;
