"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClientUser = exports.User = void 0;
const Constants_1 = require("../../utils/Constants");
const v10_1 = require("discord-api-types/v10");
const Base_1 = require("./Base");
const Channel_1 = require("./Channel");
class User extends Base_1.Base {
  /**
   * The user's username, not unique across the platform
   */
  username;
  /**
   * The user's 4-digit discord-tag
   */
  discriminator;
  /**
   * The user's avatar hash
   */
  avatar;
  /**
   * Whether the user belongs to an OAuth2 application
   */
  bot;
  /**
   * Whether the user is an Official Discord System user (part of the urgent message system)
   */
  system;
  /**
   * Whether the user has two factor enabled on their account
   */
  mfaEnabled;
  /**
   * The user's banner hash
   */
  banner;
  /**
   * The user's banner color encoded as an integer representation of hexadecimal color code
   */
  accentColor;
  /**
   * The user's chosen language option
   */
  locale;
  /**
   * Whether the email on this account has been verified
   */
  verified;
  /**
   * The flags on a user's account
   */
  flags;
  /**
   * The type of Nitro subscription on a user's account
   */
  premiumType;
  /**
   * The public flags on a user's account
   */
  publicFlags;
  /**
   * The user's display name, if it is set. For bots, this is the application name
   */
  globalName;
  /**
   * The user's avatar decoration hash
   */
  avatarDecoration;
  constructor(data) {
    super(data, data.client);
    this._update(data);
  }
  _update(data) {
    if ("username" in data) this.username = data.username;
    if ("global_name" in data) this.globalName = data.global_name;
    if ("avatar_decoration" in data)
      this.avatarDecoration = data.avatar_decoration;
    if ("discriminator" in data) this.discriminator = data.discriminator;
    if ("avatar" in data) this.avatar = data.avatar;
    if ("banner" in data) this.banner = data.banner;
    if ("bot" in data) this.bot = Boolean(data.bot);
    else this.bot ??= false;
    if ("locale" in data) this.locale = data.locale;
    if ("mfa_enabled") this.mfaEnabled = Boolean(data.mfa_enabled);
    else this.mfaEnabled ??= false;
    if ("premium_type" in data) this.premiumType = data.premium_type;
    if ("verified" in data) this.verified = Boolean(data.verified);
    else this.verified ??= false;
    if ("flags" in data && data.flags) this.flags = data.flags;
    else this.flags ??= null;
    if ("public_flags" in data && data.public_flags)
      this.publicFlags = data.public_flags;
    else this.publicFlags ??= null;
    if ("system" in data) this.system = Boolean(data.system);
    else this.system ??= false;
    if ("accent_color" in data && data.accent_color)
      this.accentColor = data.accent_color;
    else this.accentColor ??= null;
    return this;
  }
  /**
   * The user's avatar decoration url
   * @param options options for avatar decoration url
   * @returns
   */
  avatarDecorationURL(options) {
    if (!this.avatarDecoration) {
      return null;
    }
    let url =
      v10_1.RouteBases.cdn +
      v10_1.CDNRoutes.userAvatarDecoration(this.id, this.avatarDecoration);
    if (options?.size) {
      url += "?size=" + options.size.toString();
    }
    return url;
  }
  /**
   * The default user's avatar url
   * @returns
   */
  get defaultAvatarURL() {
    return (
      v10_1.RouteBases.cdn +
      v10_1.CDNRoutes.defaultUserAvatar(Number(this.discriminator) % 5)
    );
  }
  /**
   * The user's avatar url
   * @param options options for avatar url
   * @returns
   */
  avatarURL(options) {
    if (!this.avatar) {
      return null;
    }
    let url =
      v10_1.RouteBases.cdn +
      v10_1.CDNRoutes.userAvatar(
        this.id,
        this.avatar,
        options?.format ?? this.avatar.startsWith("a_")
          ? v10_1.ImageFormat.GIF
          : v10_1.ImageFormat.PNG,
      );
    if (options?.size) {
      url += "?size=" + options.size.toString();
    }
    return url;
  }
  /**
   * The user's display avatar url
   * @param options options for display avatar url
   * @returns
   */
  displayAvatarURL(options) {
    return this.avatarURL(options) ?? this.defaultAvatarURL;
  }
  /**
   * The user's display name
   * @returns
   */
  displayName() {
    return this.globalName ?? this.username;
  }
  /**
   * The user's username and discriminator
   */
  get tag() {
    return this.username + "#" + this.discriminator;
  }
  /**
   * The dm of user
   */
  get dm() {
    let dm = this._client.channels.cache.find((ch) => {
      if (!ch.isDM()) return false;
      const id = ch.userId;
      return ch.type === v10_1.ChannelType.DM && id === this.id;
    })?.[1];
    if (!dm) return null;
    if (!(dm instanceof Channel_1.DMChannel)) {
      dm = new Channel_1.DMChannel({ ...dm, client: this._client });
    }
    return dm;
  }
  /**
   * Create a new DM channel for this user
   */
  async createDM() {
    const dm = await this._client.rest.createDM(this.id);
    return this._client.channels.add(dm);
  }
  /**
   * Delete this user dm if existing one
   */
  async deleteDM() {
    if (!this.dm) return;
    await this._client.rest.deleteChannel(this.dm.id);
    this._client.channels.cache.delete(this.dm.id);
  }
  get forged() {
    return Boolean(this.id && !this.flags && !this.username);
  }
  /**
   * Update information of this user
   *
   * Util if this is forged
   * @returns
   */
  async fetchInformation() {
    const data = await this._client.rest.getUser(this.id);
    return this._update(data);
  }
  toString() {
    return (0, Constants_1.userMention)(this.id);
  }
  toJSON() {
    return Base_1.Base.toJSON(this, [
      "accentColor",
      "avatar",
      "avatarURL",
      "banner",
      "bot",
      "createdAt",
      "defaultAvatarURL",
      "discriminator",
      "dm",
      "username",
      "tag",
      "system",
      "rawData",
      "publicFlags",
      "premiumType",
      "mfaEnabled",
      "locale",
      "id",
      "flags",
    ]);
  }
}
exports.User = User;
class ClientUser extends User {
  async edit(data) {
    let avatar;
    if (data.avatar && Buffer.isBuffer(data.avatar)) {
      avatar = `data:${data.avatar.contentType};base64,${data.avatar.toString(
        "base64",
      )}`;
    }
    this._client.rest.patch(v10_1.Routes.user(), {
      username: data.username,
      avatar,
    });
  }
}
exports.ClientUser = ClientUser;
