import { DataWithClient, MessagePostData } from "../types/index";
import { APIAttachment, APIEmbed, APIMessage as RawAPIMessage, APIStickerItem, APIUser, MessageFlags as MFlags, MessageType } from "discord-api-types/v10";
import { DataCache } from "../manager/DataManager";
import { Base } from "./Base";
import { BitField } from "./BitField";
import { TextBasedChannel } from "./Channel";
import { APISuperReaction, APIReaction, Emoji, Reaction } from "./Emoji";
import { Guild } from "./Guild";
import { User } from "./User";
import { Member } from "./Member";
export declare class MessageFlags extends BitField<MFlags, typeof MFlags> {
    constructor(flags: MFlags);
    static Flags: typeof MFlags;
}
export interface APIMessage extends RawAPIMessage {
    reactions?: (APIReaction | APISuperReaction)[];
    referenced_message?: APIMessage | null;
}
export declare class Message extends Base {
    /**
     * ID of the channel the message was sent in
     */
    channelId: string;
    /**
     * The channel the message was sent in
     */
    channel: TextBasedChannel | null;
    /**
     * When this message was sent
     */
    timestamp: number;
    /**
     * Contents of the message
     */
    content: string;
    /**
     * Whether this message mentions everyone
     */
    mentionEveryone: boolean;
    /**
     * Any embedded content
     */
    embeds: APIEmbed[];
    /**
     * Any attached files
     */
    attachments: APIAttachment[];
    /**
     * The message associated with the message_reference
     */
    referencedMessage: Message | null;
    /**
     * If the message is generated by a webhook, this is the webhook's id
     */
    webhookId?: string;
    /**
     * A nonce that can be used for optimistic message sending (up to 25 characters)
     */
    nonce?: string | number;
    /**
     * The author of this message (only a valid user in the case where the message is generated by a user or bot user)
     */
    user: User | APIUser;
    /**
     * Type of message
     */
    type: MessageType;
    /**
     * A generally increasing integer (there may be gaps or duplicates) that represents the approximate position of the message in a thread
     *
     * it can be used to estimate the relative position of the message in a thread in company with total_message_sent on parent thread
     */
    position: number;
    /**
     * Message flags combined as a bitfield
     */
    flags: MessageFlags;
    /**
     * When this message was edited (or null if never)
     */
    editedTimestamp: number | null;
    /**
     * Sent if the message contains stickers
     */
    stickerItems?: APIStickerItem[];
    /**
     * Guild was message has sent
     */
    guild: Guild | null;
    /**
     * This message has resolved
     */
    isResolved: boolean;
    /**
     * Reactions in this message
     */
    reactions: DataCache<Reaction>;
    /**
     * Id of guild was message has sent
     */
    guildId?: string;
    /**
     * The member of this message (only received in guild)
     */
    member: Member | null;
    rawData: APIMessage;
    constructor(data: DataWithClient<APIMessage>, guild?: Guild);
    /**
     * Reply this message
     * @param content The content  to reply
     * @returns
     */
    reply(content: MessagePostData | string): Promise<Message | undefined>;
    /**
     * Create's a reaction in message
     * @param emoji Emoji to create te reactions
     * @returns
     */
    createReaction(emoji: string | Emoji): Promise<APIReaction | Reaction>;
    /**
     * Delete the message
     * @param reason Reason to delete message
     * @returns
     */
    delete(reason?: string): Promise<void> | undefined;
    /**
     * Edit the message
     * @param content The new content of message
     * @returns
     */
    edit(content: MessagePostData): Promise<this>;
    /**
     * Fetch's the webhook of message
     * @returns
     */
    fetchWebhook(): Promise<import("./Webhook").Webhook> | null;
    _update(data: APIMessage): this;
    toJSON(): {
        readonly createdAt: number;
        id: string;
        rawData: APIMessage;
        flags: MessageFlags;
        type: number;
        position: number;
        user: User | APIUser;
        guild: Guild | null;
        channel: TextBasedChannel | null;
        channelId: string;
        reactions: Reaction[];
        content: string;
        embeds: APIEmbed[];
        attachments: APIAttachment[];
        nonce?: string | number | undefined;
        guildId?: string | undefined;
        editedTimestamp: number | null;
        isResolved: boolean;
        mentionEveryone: boolean;
        referencedMessage: Message | null;
        stickerItems?: APIStickerItem[] | undefined;
        timestamp: number;
        webhookId?: string | undefined;
    };
}
//# sourceMappingURL=Message.d.ts.map